% *************************************************************************
% ********************************** DOER *********************************
% A Dynamic and On-line Ensemble Regression for Changing Environments
% *************************************************************************
% Version 1.0 Nov/26/2015
%
% Developed in MATLAB R2010a
% 
% Author and programmer: Symone Gomes Soares
%
% e-mail: symonesoares@gmail.com 
%
% Homepage: http://sites.google.com/site/symonesoares/
%   
% Paper: SOARES, Symone Gomes; ARAÚJO, Rui. A Dynamic and On-line Ensemble 
%        Regression for Changing Environments. Expert Systems with
%        Applications, v. 42, p. 2935-2948, 2015. 
%        DOI: http://dx.doi.org/10.1016/j.eswa.2014.11.053
%
% *************************************************************************
% *************************************************************************
% An Example of the DOER (Dynamic and On-line Ensemble Regression) 
% algorithm using the *On-line Sequential Extreme Learning Machine (OS-ELM)
% and the **FCCU data set.
%
%  *LIANG, N.-Y.; HUANG, G.-B.; SARATCHANDRAN, P.; and SUNDARARAJAN, N. 
%   (2006). A fast and accurate online sequential learning algorithm for 
%   feedforward networks. IEEE Transactions on Neural Networks, 17(6), 
%   1411–1423.
%
%
% **LIU, Y.; Hu, N.; WANG, H.; LI, P. 2009. Soft Chemical Analyzer Development Using
%   Adaptive Least-Squares Support Vector Regression With Selective Pruning and
%   Variable Moving Window Size. Ind. Eng. Chem. Res. 48 (May (12)),
%   5731–5741.
%
% *************************************************************************

clear; clc; close all;

% Loading the FCCU data set
% Using only the first output (gasoline concentration)
D      = load('FCCU.txt'); 
input  = D(:,1:6); % attributing the inputs  
output = D(:,7);   % attributing the output  

% Parameter Setting
kfold   = 10;    % number of folds for the k-fold cross-validation
sW      = 20;    % window's size (sW >= kfold)
maxNHL  = 20;    % maximum number of hidden nodes in OS-ELM (maxHL >= sW)
Alpha   = 0.04;  % factor to include a new model (0 < Alpha < 1);     
ActF    = 'sig'; % activation function for OS-ELM   
B       = 15;    % maximum number of models in the ensemble (B > 1)

% Calling fDOER fuction...
fprintf('*** Starting fDOER function ***');
[CC, MSE, RMSE, YR, YP, time] = fDOER(input, output, sW, maxNHL, Alpha, kfold, ActF, B);

fprintf('\n*** Ploting Testing Data ***\n');
figure;
plot(YR, 'k');
hold on;
plot(YP, 'r');
xlim([1 size(YR,2)]);
legend('Real Output','Predicted Output', 2);
