function [TrainAcc, TrainOutput, ELM] = OSELM_Reg_Train(P0, T0, nHiddenNeurons, ActivationFunction)

    %%%%%%%%%%% Load dataset
    nInputNeurons=size(P0,2);

    %%%%%%%%%%% step 1 Initialization Phase
    IW = rand(nHiddenNeurons,nInputNeurons)*2-1;
    switch lower(ActivationFunction)
        case{'rbf'}
            Bias = rand(1,nHiddenNeurons);
            H0 = RBFun(P0,IW,Bias);
        case{'sig'}
            Bias = rand(1,nHiddenNeurons)*2-1;
            H0 = SigActFun(P0,IW,Bias);
        case{'sin'}
            Bias = rand(1,nHiddenNeurons)*2-1;
            H0 = SinActFun(P0,IW,Bias);
        case{'hardlim'}
            Bias = rand(1,nHiddenNeurons)*2-1;
            H0 = HardlimActFun(P0,IW,Bias);
            H0 = double(H0);
    end
    M    = pinv(H0' * H0);
    beta = pinv(H0) * T0;
    
    %%%%%%%%%%% Calculate the training accuracy
    Y = H0 * beta;                            %   Y: the actual output of the training data
    TrainAcc.rmse = sqrt(mse(T0 - Y));                  %   Calculate training accuracy (RMSE) for regression case
    TrainAcc.mse  = mse(T0 - Y);                  %   Calculate training accuracy (RMSE) for regression case
    TrainAcc.cc   = corr2(T0, Y);                  %   Calculate training accuracy (RMSE) for regression case
    TrainOutput   = Y;
    clear H;
    %Giving the ELM parameters
    ELM.NumberofHiddenNeurons = nHiddenNeurons;
    ELM.ActivationFunction = ActivationFunction;
    ELM.Bias = Bias;
    ELM.beta = beta;
    ELM.IW   = IW;
    ELM.M    = M;
end
 
